@@ -368,6 +368,11 @@ gpdf_window_close (GPdfWindow *gpdf_window)
 {
 	g_return_if_fail (GPDF_IS_NON_NULL_WINDOW (gpdf_window));
 
+	bonobo_control_frame_set_autoactivate
+	  (bonobo_widget_get_control_frame (gpdf_window->priv->bonobo_widget),
+	   FALSE);
+	bonobo_control_frame_control_deactivate
+	  (bonobo_widget_get_control_frame (gpdf_window->priv->bonobo_widget));
 	gtk_widget_destroy (GTK_WIDGET (gpdf_window));
 
 	if (window_list == NULL)
@@ -518,51 +523,6 @@ gpdf_window_window_state_changed (GtkWidget *widget, GdkEventWindowState *event,
 	return FALSE;
 }
 
-static void
-file_dialog_response (GtkWidget *chooser, gint response, gboolean *dialog_result)
-{
-	gchar *file_name;
-
-	if (response != GTK_RESPONSE_OK) {
-		gtk_widget_hide (chooser);
-		gtk_main_quit ();
-		return;
-	}
-
-	file_name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
-
-	if (!g_file_test (file_name, G_FILE_TEST_EXISTS)) {
-		g_message ("!file exists in file_dialog_ok: %s", file_name);
-	} else if (g_file_test (file_name, G_FILE_TEST_IS_DIR)) {
-		/* Change into directory if that's what the user selected */
-		gchar *dir_name;
-
-		if (strlen (file_name) < 1) {
-			dir_name = g_strdup ("/");
-		} else {
-			dir_name = g_strdup (file_name);
-		}
-
-		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser),
-						     dir_name);
-		g_free (dir_name);
-	} else {
-		gtk_widget_hide (chooser);
-		gtk_main_quit ();
-		*dialog_result = TRUE;
-	}
-
-	g_free (file_name);
-}
-
-static guint
-file_dialog_delete_event (GtkWidget *widget, GdkEventAny *event)
-{
-	gtk_widget_hide (widget);
-	gtk_main_quit ();
-	return TRUE;
-}
-
 static gboolean
 gw_ask_for_uri (GPdfWindow *gpdf_window, gchar ***uri)
 {
@@ -582,20 +542,9 @@ gw_ask_for_uri (GPdfWindow *gpdf_window, gchar ***uri)
 	gtk_file_chooser_set_select_multiple (chooser, TRUE); 
 	gtk_window_set_modal (GTK_WINDOW (chooser), TRUE);
 
-	/* Connect the signals for Ok and Cancel */
-	g_signal_connect (G_OBJECT (chooser), "response",
-			  G_CALLBACK (file_dialog_response), &accepted);
-
 	gtk_window_set_position (GTK_WINDOW (chooser), GTK_WIN_POS_MOUSE);
 
-	/* Make sure that we quit the main loop if the window is destroyed */
-	g_signal_connect (G_OBJECT (chooser), "delete_event",
-			  G_CALLBACK (file_dialog_delete_event), NULL);
-
-	/* Run the dialog */
-	gtk_widget_show (GTK_WIDGET (chooser));
-	gtk_grab_add (GTK_WIDGET (chooser));
-	gtk_main ();
+	accepted = (gtk_dialog_run (GTK_DIALOG (chooser)) == GTK_RESPONSE_OK);
 
 	if (accepted) {
 		GSList *files, *iter;
