@@ -83,7 +83,6 @@ struct _GPdfBookmarksViewPrivate {
 };
 
 static void	gpdf_bookmarks_view_class_init	      (GPdfBookmarksViewClass*); 
-static void	gpdf_bookmarks_view_class_finalize    (gpointer, gpointer); 
 static void	gpdf_bookmarks_view_dispose	      (GObject*); 
 static void	gpdf_bookmarks_view_finalize	      (GObject*); 
 static void	gpdf_bookmarks_view_instance_init     (GPdfBookmarksView*);
@@ -117,6 +116,7 @@ enum {
 
 enum {
 	BOOKMARK_SELECTED_SIGNAL = 0,
+	READY_SIGNAL, 
 	LAST_SIGNAL
 };
 
@@ -176,6 +176,16 @@ gpdf_bookmarks_view_class_init (GPdfBookmarksViewClass *klass)
 	    NULL, NULL,
 	    gpdf_marshal_VOID__POINTER,
 	    G_TYPE_NONE, 1, G_TYPE_POINTER);
+
+	gpdf_bookmarks_view_signals [READY_SIGNAL] =
+	  g_signal_new (
+	    "ready",
+	    G_TYPE_FROM_CLASS (gobject_class),
+	    G_SIGNAL_RUN_LAST,
+	    G_STRUCT_OFFSET (GPdfBookmarksViewClass, ready),
+	    NULL, NULL,
+	    gpdf_marshal_VOID__VOID,
+	    G_TYPE_NONE, 0);
 }
 
 static void
@@ -188,11 +198,11 @@ gpdf_bookmarks_view_emit_bookmark_selected (GPdfBookmarksView *bookmarks_view,
 }
 
 static void
-gpdf_bookmarks_view_init (GPdfBookmarksView *bookmarks_view)
+gpdf_bookmarks_view_emit_ready (GPdfBookmarksView *bookmarks_view)
 {
-  GtkScrolledWindow *window;
-
-  window = GTK_SCROLLED_WINDOW (bookmarks_view);
+	g_signal_emit (G_OBJECT (bookmarks_view),
+		       gpdf_bookmarks_view_signals [READY_SIGNAL],
+		       0);
 }
 
 /*
@@ -225,7 +235,7 @@ gpdf_bookmarks_view_construct (GPdfBookmarksView *bookmarks_view)
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (bookmarks_view),
 					GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (bookmarks_view),
-					     GTK_SHADOW_IN);
+					     GTK_SHADOW_NONE);
 	
 	/* Exprim interrest in page changed signal */
 	g_signal_connect (G_OBJECT (priv->gpdf_view), "page_changed",
@@ -402,7 +412,6 @@ gpdf_bookmarks_view_update_bkvisual (GtkTreeModel *model,
 	GValue page_nr_value = {0, };
 	gint page_nr; 
 	GdkPixbuf *bookmark_pixbuf = NULL;
-	GtkTreeSelection  *selection;
 	gboolean page_displayed, is_expanded; 
 	
 	bookmarks_view = GPDF_BOOKMARKS_VIEW (data);
@@ -596,6 +605,22 @@ unicode_to_char (OutlineItem *outline_item,
 	return g_strdup (gstr.getCString ());
 }
 
+static gboolean
+gpdf_bookmarks_view_have_outline_items (GPdfBookmarksView *bookmarks_view)
+{
+	GPdfBookmarksViewPrivate *priv;
+
+	g_return_val_if_fail
+	  (GPDF_IS_NON_NULL_BOOKMARKS_VIEW (bookmarks_view),
+	   FALSE);
+	
+	priv = bookmarks_view->priv;
+	
+	return (priv->outlines != NULL &&
+		priv->outlines->getItems () !=  NULL &&
+		priv->outlines->getItems ()->getLength () > 0);
+}
+
 static void
 gpdf_bookmarks_view_update_popup_actions (GPdfBookmarksView *bookmarks_view)
 {
@@ -609,7 +634,10 @@ gpdf_bookmarks_view_update_popup_actions (GPdfBookmarksView *bookmarks_view)
 	gboolean is_expandable = FALSE, is_expanded = FALSE;
 
 	g_return_if_fail (GPDF_IS_NON_NULL_BOOKMARKS_VIEW (bookmarks_view));
-    
+
+	if (!gpdf_bookmarks_view_have_outline_items (bookmarks_view))
+	  return;
+	
 	priv = bookmarks_view->priv;
 	model = GTK_TREE_MODEL (priv->model); 
 
@@ -629,10 +657,12 @@ gpdf_bookmarks_view_update_popup_actions (GPdfBookmarksView *bookmarks_view)
 	}
 
 	item = priv->popup_menu_uiinfo[POPUP_MENU_EXPAND_INDEX].widget;
-	gtk_widget_set_sensitive (item, is_expandable && !is_expanded); 
+	if (GTK_IS_WIDGET (item))
+	  gtk_widget_set_sensitive (item, is_expandable && !is_expanded); 
 	
 	item = priv->popup_menu_uiinfo[POPUP_MENU_COLLAPSE_INDEX].widget;
-	gtk_widget_set_sensitive (item, is_expandable && is_expanded);	
+	if (GTK_IS_WIDGET (item))
+	  gtk_widget_set_sensitive (item, is_expandable && is_expanded);	
 }
 
 static gboolean
@@ -915,19 +945,42 @@ gpdf_bookmarks_view_expand_unique_root (GPdfBookmarksView *bookmarks_view)
 	}
 }
 
+static gboolean
+gpdf_bookmarks_view_populate_idle (GPdfBookmarksView *bookmarks_view)
+{
+	GPdfBookmarksViewPrivate *priv;
+	GtkTreeSelection *selection; 
+	
+	g_return_val_if_fail (GPDF_IS_NON_NULL_BOOKMARKS_VIEW (bookmarks_view), TRUE);
+	
+	priv = bookmarks_view->priv;
+
+	/* Create bookmarks tree */
+	if (gpdf_bookmarks_view_have_outline_items (bookmarks_view)) {
+		
+		gpdf_bookmarks_view_update_bookmarks_tree (bookmarks_view);
+		
+		/* Add listener for popup actions sensitivity */
+		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview)); 
+		priv->selection_id =
+		  g_signal_connect (G_OBJECT (selection), "changed",
+				    G_CALLBACK (gpdf_bookmarks_view_selection_changed_cb), 
+				    bookmarks_view);
+		
+		/* If root is unique, let's expand it */
+		gpdf_bookmarks_view_expand_unique_root (bookmarks_view);
+	}
+
+	gpdf_bookmarks_view_emit_ready (bookmarks_view);
+
+	return FALSE;
+}
+
 void
 gpdf_bookmarks_view_set_pdf_doc (GPdfBookmarksView *bookmarks_view,
 				 PDFDoc *pdf_doc)
 {
 	GPdfBookmarksViewPrivate *priv;
-	gint num_pages;
-	gint page;
-	Page *the_page;
-	Object *the_thumb;
-	Object obj;
-	gint width, height, bitsPerComponent;
-	GString *colorSpace;
-	GtkTreeSelection *selection; 
 	
 	g_return_if_fail (GPDF_IS_NON_NULL_BOOKMARKS_VIEW (bookmarks_view));
 	
@@ -939,30 +992,8 @@ gpdf_bookmarks_view_set_pdf_doc (GPdfBookmarksView *bookmarks_view,
 		priv->outlines = pdf_doc->getOutline ();
 		/* Current page set through 'page_changed' signal */
 		
-		/* Create bookmarks tree */
-		if (priv->outlines->getItems ())
-			gpdf_bookmarks_view_update_bookmarks_tree (bookmarks_view);
-		else
-		{
-			GtkWidget *item;
-			
-			/* Disable all tools menu items (expand/collapse) */
-			item = priv->popup_menu_uiinfo[POPUP_MENU_EXPAND_ALL_INDEX].widget;
-			gtk_widget_set_sensitive (item, FALSE);
-
-			item = priv->popup_menu_uiinfo[POPUP_MENU_COLLAPSE_ALL_INDEX].widget;
-			gtk_widget_set_sensitive (item, FALSE);
-		}
-		
-		/* Add listener for popup actions sensitivity */
-		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview)); 
-		priv->selection_id =
-		  g_signal_connect (G_OBJECT (selection), "changed",
-				    G_CALLBACK (gpdf_bookmarks_view_selection_changed_cb), 
-				    bookmarks_view);
-		
-		/* If root is unique, let's expand it */
-		gpdf_bookmarks_view_expand_unique_root (bookmarks_view); 
+		g_idle_add ((GSourceFunc)gpdf_bookmarks_view_populate_idle,
+			    bookmarks_view);
 	}
 }
 
@@ -1057,6 +1088,9 @@ gpdf_bookmarks_view_get_tools_menu (GPdfBookmarksView *bookmarks_view)
 	g_return_val_if_fail (GPDF_IS_NON_NULL_BOOKMARKS_VIEW (bookmarks_view), NULL);
 	
 	priv = bookmarks_view->priv;
+
+	if (!gpdf_bookmarks_view_have_outline_items (bookmarks_view))
+	  return NULL;
 	
 	if (!priv->popup_menu) {
 		priv->popup_menu = gtk_menu_new ();
