@@ -21,24 +21,34 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include <unistd.h>
 #include <aconf.h>
 #include <math.h>
 #include "gpdf-view.h"
 #include "gpdf-util.h"
 #include "gpdf-marshal.c"
 #include "gpdf-g-switch.h"
+#  include <glib/gi18n.h>
 #  include <gdk/gdkkeysyms.h>
 #  include <libgnome/gnome-macros.h>
-#  include <libgnome/gnome-i18n.h>
 #  include <libgnomeprint/gnome-print.h>
 #  include <libgnomeprintui/gnome-print-preview.h>
+#  include <libgnomevfs/gnome-vfs-mime.h>
+#  include <libgnomevfs/gnome-vfs-mime-handlers.h>
+#  include <gtk/gtkwidget.h>
 #  include <ggv-document.h>
 #  include "gtkgesture.h"
+#  include <glade/glade.h>
 #include "gpdf-g-switch.h"
+#include "gpdf-control.h"
+#include "gpdf-control-private.h"
 #include "GPOutputDev.h"
 #include "PDFDoc.h"
 #include "Link.h"
 #include "gpdf-links-canvas-layer.h"
+#ifdef USE_ANNOTS_VIEW
+#include "gpdf-annots-view.h"
+#endif
 
 BEGIN_EXTERN_C
 
@@ -63,6 +73,12 @@ struct _GPdfViewPrivate {
 
 	G_List *forward_history;
 	G_List *backward_history;
+
+#ifdef USE_ANNOTS_VIEW
+	gboolean (*annots_cb)(Annot *annot, gpointer data);
+	GPdfAnnotsView *annots_view;
+#endif
+	GPdfControl *parent;
 };
 
 #define PARENT_TYPE GNOME_TYPE_CANVAS
@@ -83,7 +99,7 @@ enum {
 static guint gpdf_view_signals [LAST_SIGNAL];
 
 
-#define PAGE_PAD 3
+#define PAGE_PAD ((gdouble)3.0)
 
 static void
 gpdf_view_page_background_show (GPdfView *gpdf_view)
@@ -117,10 +133,26 @@ gpdf_view_page_background_resize (GPdfView *gpdf_view,
 			       NULL);
 }
 
+#ifdef USE_ANNOTS_VIEW
+static GPdfAnnotsView *
+gpdf_view_get_annots_view (GPdfView *gpdf_view)
+{
+	g_return_val_if_fail (GPDF_IS_NON_NULL_VIEW (gpdf_view),
+			      NULL);
+
+	if (!gpdf_view->priv->annots_view)
+	  gpdf_view->priv->annots_view = GPDF_ANNOTS_VIEW 
+	    (gpdf_control_private_get_annots_view (gpdf_view->priv->parent)); 
+
+	return gpdf_view->priv->annots_view; 
+}
+#endif
+
 static void
-setup_page_transform (gdouble *transform,
-		      gdouble x1, gdouble y1, gdouble x2, gdouble y2,
-		      gint angle)
+gpdf_view_setup_page_transform (gdouble *transform,
+				gdouble x1, gdouble y1,
+				gdouble x2, gdouble y2,
+				gint angle)
 {
 	double width;
 	double height;
@@ -154,7 +186,7 @@ setup_page_transform (gdouble *transform,
 }
 
 static gint
-canonical_multiple_of_90 (gint n)
+gpdf_view_canonical_multiple_of_90 (gint n)
 {
 	while (n >= 360) {
 		n -= 360;
@@ -191,6 +223,8 @@ gpdf_view_history_stack_page (GPdfView *gpdf_view)
 	hnode = (GPdfViewHistory *)g_malloc (sizeof(GPdfViewHistory));
 	hnode->zoom = priv->zoom;
 	hnode->page = priv->current_page;
+	hnode->hval = gtk_adjustment_get_value(GTK_LAYOUT (gpdf_view)->hadjustment); 
+	hnode->vval = gtk_adjustment_get_value(GTK_LAYOUT (gpdf_view)->vadjustment); 
 	priv->backward_history = g_list_prepend (priv->backward_history, (gpointer)hnode);
 }
 
@@ -291,8 +325,83 @@ gpdf_view_link_action_named (GPdfView *gpdf_view, LinkNamed *link_named)
 }
 
 static void
-link_clicked_cb (GPdfLinksCanvasLayer *links_layer, Link *link,
-		 gpointer user_data)
+gpdf_view_link_action_uri (GPdfView *gpdf_view, LinkURI *link_uri)
+{
+	GString *uri; 
+	GnomeVFSMimeAction *action; 
+	GnomeVFSMimeApplication *app;
+	GnomeVFSURI *vfsuri; 
+	G_List *uris = NULL;
+	const gchar *mime;
+
+	g_message ("gpdf_view_link_action_uri: enters ..."); 
+
+	g_return_if_fail (GPDF_IS_VIEW_WITH_DOC (gpdf_view));
+	g_return_if_fail (dynamic_cast <LinkURI *> (link_uri) != NULL);
+
+	uri = link_uri->getURI (); 
+	g_message ("gpdf_view_link_action_uri: uri = '%s'", (gchar *)uri->getCString ()); 
+	uris = g_list_append (uris, (gchar *)uri->getCString ());
+
+	vfsuri = gnome_vfs_uri_new ((const gchar *)uris->data); 
+	mime = gnome_vfs_get_mime_type_common (vfsuri);
+
+	g_message ("gpdf_view_link_action_uri: mime = '%s'", mime); 
+
+	if (!mime) {
+		mime = "text/html";
+		g_message ("gpdf_view_link_action_uri: using default mime = '%s'", mime); 
+	}
+
+	action = gnome_vfs_mime_get_default_action (mime); 
+	if (gnome_vfs_mime_action_launch (action, uris) != GNOME_VFS_OK)
+	{
+		app = gnome_vfs_mime_get_default_application (mime); 
+		
+		if (!app || gnome_vfs_mime_application_launch (app, uris) != GNOME_VFS_OK) {
+
+			/* Fallback: No component or app was found. Use a browser */
+			mime = "text/html";
+			action = gnome_vfs_mime_get_default_action (mime);
+			if (gnome_vfs_mime_action_launch (action, uris) != GNOME_VFS_OK)
+			{
+				app = gnome_vfs_mime_get_default_application (mime); 
+				if (!app || gnome_vfs_mime_application_launch (app, uris) != GNOME_VFS_OK) {
+			
+					gchar *msg = 
+					  g_strdup_printf(_("Failed to launch %s for URI: %s"),
+							  app->name,
+							  (const gchar *)uris->data);
+					gpdf_control_private_error_dialog (gpdf_view->priv->parent, 
+									   _("Application Launch Failure!"),
+									   msg,
+									   TRUE);
+					g_free (msg);
+				}
+			}
+		}
+		else {
+			gchar *msg = g_strdup_printf ("URI '%s' launched with %s",
+						      (const gchar *)uris->data,
+						      app->name);
+			gpdf_control_private_info_dialog (gpdf_view->priv->parent, 
+							  _("Application Launch notification!"),
+							  msg,
+							  TRUE);
+			g_free (msg);			
+		}
+
+		gnome_vfs_mime_application_free (app); 
+	}
+	g_free (uris); 
+	gnome_vfs_uri_unref (vfsuri); 
+	delete uri; 
+}
+
+static void
+gpdf_view_link_clicked_cb (GPdfLinksCanvasLayer *links_layer,
+			   Link                 *link,
+			   gpointer              user_data)
 {
 	GPdfView *gpdf_view;
 	LinkAction *link_action;
@@ -312,6 +421,10 @@ link_clicked_cb (GPdfLinksCanvasLayer *links_layer, Link *link,
 		gpdf_view_link_action_named (
 			gpdf_view, dynamic_cast <LinkNamed *> (link_action));
 		break;
+	case actionURI:
+		gpdf_view_link_action_uri (
+			gpdf_view, dynamic_cast <LinkURI *> (link_action));
+		break; 
 	default:
 		g_message ("Click on link: unimplemented Action");
 		break;
@@ -340,6 +453,8 @@ gpdf_view_render_page (GPdfView *gpdf_view)
 	if (priv->requested_page == priv->current_page)
 		return FALSE;
 
+	gpdf_control_private_set_wait_cursor (priv->parent);
+	
 	page = priv->requested_page;
 
 	if (priv->print_context) {
@@ -351,11 +466,11 @@ gpdf_view_render_page (GPdfView *gpdf_view)
 	pdf_page = pdf_doc->getCatalog ()->getPage (page);
 	rect = pdf_page->getBox ();
 
-	page_rotate = canonical_multiple_of_90 (pdf_doc->getPageRotate (page));
+	page_rotate = gpdf_view_canonical_multiple_of_90 (pdf_doc->getPageRotate (page));
 
-	setup_page_transform (page_transform,
-			      rect->x1, rect->y1, rect->x2, rect->y2,
-			      page_rotate);
+	gpdf_view_setup_page_transform (page_transform,
+					rect->x1, rect->y1, rect->x2, rect->y2,
+					page_rotate);
 
 	if (page_rotate == 90 || page_rotate == 270) {
 		page_width_rot = pdf_doc->getPageHeight (page);
@@ -379,11 +494,24 @@ gpdf_view_render_page (GPdfView *gpdf_view)
 	gnome_print_config_unref (config);
 
 	priv->output_dev->setPrintContext (priv->print_context);
+#ifdef USE_ANNOTS_VIEW
 	priv->pdf_doc->displayPage (priv->output_dev,
 				    page,
 				    72,
 				    0,
-				    gFalse);
+				    gFalse,
+				    NULL, NULL, 
+				    priv->annots_cb,
+				    gpdf_view_get_annots_view (gpdf_view));
+#else
+	priv->pdf_doc->displayPage (priv->output_dev,
+				    page,
+				    72,
+				    0,
+				    gFalse,
+				    NULL, NULL, 
+				    NULL, NULL);
+#endif
 
 	priv->current_page = page;
 
@@ -408,9 +536,13 @@ gpdf_view_render_page (GPdfView *gpdf_view)
 	gnome_canvas_item_affine_relative (
 		GNOME_CANVAS_ITEM (priv->links_layer), page_transform);
 
-	g_signal_connect (G_OBJECT (priv->links_layer), "link_clicked",
-			  G_CALLBACK (link_clicked_cb), gpdf_view);
+	gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (priv->links_layer));
 
+	g_signal_connect (G_OBJECT (priv->links_layer), "link_clicked",
+			  G_CALLBACK (gpdf_view_link_clicked_cb),
+			  gpdf_view);
+
+	gpdf_control_private_reset_cursor (priv->parent);
 	return FALSE;
 }
 
@@ -469,6 +601,17 @@ gpdf_view_scroll_to_bottom (GPdfView *gpdf_view)
 				 - GTK_WIDGET (gpdf_view)->allocation.height));
 }
 
+gint
+gpdf_view_get_current_page (GPdfView *gpdf_view)
+{
+	GPdfViewPrivate *priv;
+	
+	g_return_val_if_fail (GPDF_IS_VIEW_WITH_DOC (gpdf_view), 0);
+	priv = gpdf_view->priv;
+	return priv->requested_page != priv->current_page ?
+	  priv->requested_page : priv->current_page; 
+}
+
 void
 gpdf_view_page_prev (GPdfView *gpdf_view)
 {
@@ -533,6 +676,7 @@ gpdf_view_back_history (GPdfView *gpdf_view)
 	GPdfViewHistory *hnode;
 	int target_page = 0;
 	double target_zoom = (double)0.0;
+	double target_hval, target_vval; 
 
 	g_return_if_fail (GPDF_IS_VIEW_WITH_DOC (gpdf_view));
 	priv = gpdf_view->priv;
@@ -543,6 +687,8 @@ gpdf_view_back_history (GPdfView *gpdf_view)
 	hnode = (GPdfViewHistory *)priv->backward_history->data;
 	target_page = hnode->page;
 	target_zoom = hnode->zoom;
+	target_hval = hnode->hval; 
+	target_vval = hnode->vval; 
 	priv->backward_history = g_list_remove (priv->backward_history,
 						(gpointer)hnode);
 	if (!g_list_length (priv->backward_history)) {
@@ -551,11 +697,14 @@ gpdf_view_back_history (GPdfView *gpdf_view)
 	}
 	hnode->page = priv->current_page;
 	hnode->zoom = priv->zoom;
+	hnode->hval = gtk_adjustment_get_value(GTK_LAYOUT (gpdf_view)->hadjustment); 
+	hnode->vval = gtk_adjustment_get_value(GTK_LAYOUT (gpdf_view)->vadjustment); 
 	priv->forward_history = g_list_prepend (priv->forward_history, hnode);
 
 	gpdf_view_goto_page_no_history (gpdf_view, target_page);
-	/* TODO: change zoom level and scroll offset if different */
-	gpdf_view_scroll_to_top (gpdf_view);
+	gpdf_view_zoom (gpdf_view, target_zoom, FALSE); 
+	gtk_adjustment_set_value(GTK_LAYOUT (gpdf_view)->hadjustment, target_hval);
+	gtk_adjustment_set_value(GTK_LAYOUT (gpdf_view)->vadjustment, target_vval);
 }
 
 void
@@ -565,6 +714,7 @@ gpdf_view_forward_history (GPdfView *gpdf_view)
 	GPdfViewHistory *hnode;
 	int target_page = 0;
 	double target_zoom = (double)0.0;
+	double target_hval, target_vval; 
 
 	g_return_if_fail (GPDF_IS_VIEW_WITH_DOC (gpdf_view));
 	priv = gpdf_view->priv;
@@ -574,6 +724,8 @@ gpdf_view_forward_history (GPdfView *gpdf_view)
 	hnode = (GPdfViewHistory *)priv->forward_history->data;
 	target_page = hnode->page;
 	target_zoom = hnode->zoom;
+	target_hval = hnode->hval; 
+	target_vval = hnode->vval; 
 	priv->forward_history = g_list_remove (priv->forward_history,
 					       (gpointer)hnode);
 	if (!g_list_length (priv->forward_history)) {
@@ -582,11 +734,14 @@ gpdf_view_forward_history (GPdfView *gpdf_view)
 	}
 	hnode->page = priv->current_page;
 	hnode->zoom = priv->zoom;
+	hnode->hval = gtk_adjustment_get_value(GTK_LAYOUT (gpdf_view)->hadjustment); 
+	hnode->vval = gtk_adjustment_get_value(GTK_LAYOUT (gpdf_view)->vadjustment); 
 	priv->backward_history = g_list_prepend (priv->backward_history, hnode);
 
 	gpdf_view_goto_page_no_history (gpdf_view, target_page);
-	/* TODO: change zoom level and scroll offset if different */
-	gpdf_view_scroll_to_top (gpdf_view);
+	gpdf_view_zoom (gpdf_view, target_zoom, FALSE); 
+	gtk_adjustment_set_value(GTK_LAYOUT (gpdf_view)->hadjustment, target_hval);
+	gtk_adjustment_set_value(GTK_LAYOUT (gpdf_view)->vadjustment, target_vval);
 }
 
 gboolean
@@ -699,14 +854,11 @@ gpdf_view_zoom_out (GPdfView *gpdf_view)
 void
 gpdf_view_zoom_fit (GPdfView *gpdf_view, gint width, gint height)
 {
-	GPdfViewPrivate *priv;
 	gdouble x1, y1, x2, y2;
 	gdouble page_w, page_h, zoom_w, zoom_h;
 
 	g_return_if_fail (GPDF_IS_VIEW_WITH_DOC (gpdf_view));
 
-	priv = gpdf_view->priv;
-
 	gnome_canvas_get_scroll_region (GNOME_CANVAS (gpdf_view),
 					&x1, &y1, &x2, &y2);
 
@@ -725,14 +877,11 @@ gpdf_view_zoom_fit (GPdfView *gpdf_view, gint width, gint height)
 void
 gpdf_view_zoom_fit_width (GPdfView *gpdf_view, gint width)
 {
-	GPdfViewPrivate *priv;
 	gdouble x1, y1, x2, y2;
 	gdouble page_w;
 
 	g_return_if_fail (GPDF_IS_VIEW_WITH_DOC (gpdf_view));
 
-	priv = gpdf_view->priv;
-
 	gnome_canvas_get_scroll_region (GNOME_CANVAS (gpdf_view),
 					&x1, &y1, &x2, &y2);
 	page_w = x2 - x1 + 0.5;
@@ -742,6 +891,59 @@ gpdf_view_zoom_fit_width (GPdfView *gpdf_view, gint width)
 	gpdf_view_zoom (gpdf_view, width / page_w, FALSE);
 }
 
+void
+gpdf_view_thumbnail_selected (GPdfView   *gpdf_view,
+			      gint x, gint y,
+			      gint w, gint h,
+			      gint page)
+{
+	gpdf_view_goto_page(gpdf_view, page);
+}
+
+void
+gpdf_view_annotation_selected(GPdfView *gpdf_view,
+			      Annot    *annot, 
+			      gint      page)
+{
+	gpdf_view_goto_page (gpdf_view, page);
+}
+
+void
+gpdf_view_annotation_toggled(GPdfView   *gpdf_view,
+			     Annot      *annot, 
+			     gint        page,
+			     gboolean	 active)
+{
+	if (gpdf_view->priv->current_page != page)
+	  gpdf_view_goto_page (gpdf_view, page);
+	else {
+		gpdf_view->priv->requested_page = page; 
+		gpdf_view->priv->current_page = 0; 
+		gpdf_view_render_page (gpdf_view);
+	}
+}
+
+void
+gpdf_view_bookmark_selected (GPdfView   *gpdf_view,
+			     LinkAction *link_action)
+{
+	g_return_if_fail (GPDF_IS_VIEW (gpdf_view));
+
+	switch (link_action->getKind ()) {
+	case actionGoTo:
+		gpdf_view_link_action_goto (
+			gpdf_view, dynamic_cast <LinkGoTo *> (link_action));
+		break;
+	case actionNamed:
+		gpdf_view_link_action_named (
+			gpdf_view, dynamic_cast <LinkNamed *> (link_action));
+		break;
+	default:
+		g_message ("Bookmark selection: unimplemented Action");
+		break;
+	}
+}
+
 
 static gboolean
 gpdf_view_key_press_event (GtkWidget *widget, GdkEventKey *event)
@@ -903,6 +1105,7 @@ gpdf_view_button_press_event (GtkWidget *widget, GdkEventButton *event)
 				  NULL, cursor, event->time);
 		gdk_cursor_unref (cursor);
 		return TRUE;
+		
 	default:
 		return FALSE;
 	}
@@ -1048,11 +1251,11 @@ gpdf_view_save_as (GPdfView *gpdf_view, const gchar *pathname)
 
 	priv = gpdf_view->priv;
 
-/*	gpdf_control_private_set_wait_cursor (priv->parent); */
+	gpdf_control_private_set_wait_cursor (priv->parent);
 	
 	priv->pdf_doc->saveAs (filename);
 	
-/*	gpdf_control_private_reset_cursor (priv->parent); */
+	gpdf_control_private_reset_cursor (priv->parent);
 }
 
 
@@ -1084,6 +1287,12 @@ gpdf_view_dispose (GObject *object)
 		gtk_gesture_handler_destroy (priv->gesture_handler);
 		priv->gesture_handler = NULL;
 	}
+#ifdef USE_ANNOTS_VIEW
+	if (priv->annots_view) {
+		gtk_widget_unref (GTK_WIDGET (priv->annots_view));
+		priv->annots_view = NULL; 
+	}
+#endif
 
 	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (object));
 }
@@ -1238,8 +1447,10 @@ gpdf_view_setup_page_background (GPdfView *gpdf_view)
 	priv->page_shadow = gnome_canvas_item_new (
 		gnome_canvas_root (GNOME_CANVAS (gpdf_view)),
 		GNOME_TYPE_CANVAS_RECT,
-		"x1", PAGE_PAD - 1.0, "y1", (gdouble)PAGE_PAD - 1.0,
-		"x2", 72.0 + PAGE_PAD - 1.0, "y2", 72.0 + PAGE_PAD - 1.0,
+		"x1", (gdouble)PAGE_PAD -1.0,
+		"y1", (gdouble)PAGE_PAD -1.0,
+		"x2", 72.0 + (gdouble)PAGE_PAD -1.0,
+		"y2", 72.0 + (gdouble)PAGE_PAD -1.0,
 		"fill_color", "black", NULL);
 
 	gnome_canvas_item_lower_to_bottom (priv->page_shadow);
@@ -1262,17 +1473,22 @@ gpdf_view_construct (GPdfView *gpdf_view)
 	priv->output_dev = new GPOutputDev ();
 	priv->zoom = 1.0;
 
+#ifdef USE_ANNOTS_VIEW
+	priv->annots_cb = gpdf_annots_view_is_annot_displayed;
+#endif
+	
 	return gpdf_view;
 }
 
 GtkWidget *
-gpdf_view_new (void)
+gpdf_view_new (GPdfControl *control)
 {
 	GPdfView *gpdf_view;
 
 	gpdf_view = GPDF_VIEW (g_object_new (GPDF_TYPE_VIEW,
 					     "aa", TRUE,
 					     NULL));
+	gpdf_view->priv->parent = control; 
 
 	return GTK_WIDGET (gpdf_view_construct (gpdf_view));
 }
@@ -1283,6 +1499,7 @@ gpdf_view_class_init_trampoline (gpointer klass, gpointer data)
 	parent_class = (GnomeCanvasClass *)g_type_class_ref (PARENT_TYPE);
 	gpdf_view_class_init ((GPdfViewClass *)klass);
 }
+
 GType
 gpdf_view_get_type (void)
 {
